from colorful import Colorful
from findme import FindMe
from stylish import Stylish

# From Colorful: simple functions that provide the additional benefit of colours when viewing the data.
"""Simple function to print a list with the added bonus of ability to add colors and highlights"""
heading = "Important"
stuff = ["Super", "Cool", "Stuff"]

"""Use it to print a list"""
ls = Colorful()
ls.listme(stuff)

"""Print the list in color with a heading"""
c = Colorful(fg=213)
c.listme(stuff, heading="Amazingly", heading_style=c)

"""Print the list in color (or not) with different colors for the items vs the heading."""
h = Colorful(fg=56, bg=1, bo=True)
c = Colorful()
c.listme(stuff, heading=heading, heading_style=h)


template = "Once upon a time there was a {noun} who really wanted to {desire}. Everybody knew it would take {verb} and {stuff}..."
doc2 = "Some other stuff here {text} and there is also a {doc}, and {someone} said they really wanted to go outside."


wrap = Stylish(
    noun=" doggo ",
    desire=" jump around ",
    verb=" a joyous attitude ",
    stuff=" heaps of time "
)
wrap.fg = 123
wrap.bg = 129
wrap.it = True
wrap.findme(template)

styled = FindMe(
    text="Named",
    doc="Excellent Book",
    someone="Unnamed"
)
styled.fg = 201
styled.it = True
styled.bo = True
styled.showme(doc2)

s = Colorful(bg=69, it=True)
s.write("This is a colorful highlighted message.")

c = Colorful(fg=195, it=True, bo=True)
c.write("Here is an example that has other colors.")



# TODO >> need to organise the code because right now the classes Colorful and Stylish seems to overlap, so need some structure later when I write more functions with these. Use colorful for colour, stylish for style. But then the FindMe should be elsewhere...

# Example Code
heading = " DOGGOS "
dogs = ["Ziggy", "Bobby", "Dracula", "Leopold"]
d = FindMe()
c = Colorful(fg=204, it=True)
h = Colorful(fg=195, bg=204, bo=True)
heading = h.styleme(heading)
d.listme([c.styleme(dog) for dog in dogs], heading=heading)

nn = FindMe()
nn.fg=47
nn.bg=96
nn.listme(dogs, heading=heading, heading_style=h)


v = FindMe()
v.listme(["here", "we", "have", "dogs", "cats", 1, 4, "teddy"])
j = Colorful(fg=209, it=True)
li = ["here", "we", "have", "dogs", "cats", 1, 4, "teddy"]
bob = [j.styleme(str(item)) for item in li]
v.listme(bob)

m = FindMe()
m.fg = 24

mm = FindMe()
mm.fg = 199
mm.bg=57
mm.bo = True
m.listme(li, heading=" SOME LIST ", heading_style=mm)



# Colorful: simple functions that provide the additional benefit of colours when viewing the data.

from dataclasses import dataclass


class ColorfulConfig:
    def __init__(self, *, default):
        self._default = default

    def __set_name__(self, owner, name):
        self._name = "_" + name

    def __get__(self, obj, type):
        if obj is None:
            return self._default

        return getattr(obj, self._name, self._default)

    def __set__(self, obj, value):
        setattr(obj, self._name, (value))


@dataclass
class Colorful:
    fg: ColorfulConfig = ColorfulConfig(default=None)
    bg: ColorfulConfig = ColorfulConfig(default=None)
    bo: ColorfulConfig = ColorfulConfig(default=False)
    ft: ColorfulConfig = ColorfulConfig(default=False)
    it: ColorfulConfig = ColorfulConfig(default=False)
    ul: ColorfulConfig = ColorfulConfig(default=False)
    st: ColorfulConfig = ColorfulConfig(default=False)
    sh: ColorfulConfig = ColorfulConfig(default=False)


    def write(self, words:str):
        # TODO update to also be able to print lists, dict etc etc.
        """Functions exactly like a normal print statement with the additional bonus
        of being able to set the style and colours."""
        styling = []
        fg = []
        bg = []
        styles = [(self.it, 3), (self.bo, 1), (self.ul, 4), (self.st, 9), (self.sh, 7)]
        colours = [(self.fg, self.bg)]
        for key, value in styles:
            if key:
                s = f"\033[{value};5m"
                styling.append(s)

        for key, value in colours:
            if key:
                f = f"\033[38;5;{key}m"
                fg.append(f)
            if value:
                v = f"\033[48;5;{value}m"
                bg.append(v)
        z = "\033[0m"

        s1 = styling + fg + bg
        styled_text = "".join(s1) + words + z
        print(styled_text)

    def styleme(self, words:str):
        """Exactly the same as write(), but simply return the styled words
        so they can be used in further manipulations."""
        styling = []
        fg = []
        bg = []
        styles = [(self.it, 3), (self.bo, 1), (self.ul, 4), (self.st, 9), (self.sh, 7)]
        colours = [(self.fg, self.bg)]
        for key, value in styles:
            if key:
                s = f"\033[{value};5m"
                styling.append(s)

        for key, value in colours:
            if key:
                f = f"\033[38;5;{key}m"
                fg.append(f)
            if value:
                v = f"\033[48;5;{value}m"
                bg.append(v)
        z = "\033[0m"

        s1 = styling + fg + bg
        styled_text = "".join(s1) + words + z
        return(styled_text)

    def listme(self, list:list, heading=None, heading_style=None):
        v = [self.styleme(str(item)) for item in list]
        if not heading and not heading_style:
            print(f"{"\n".join(v)}")
        elif not heading:
            print(f"{"\n".join(v)}")
        elif heading and heading_style:
            print(f"{heading_style.styleme(str(heading))}\033[0m\n{"\n".join(v)}")
        elif heading and not heading_style:
            print(f"{heading}\033[0m\n{"\n".join(v)}")

        else:
            print("You might want to check you have configured the colors and styles.")

        def reverseme(self, words):
            pass


if __name__ == "__main__":

    items = ["Enjoy the", "simplicity"]
    c = Colorful(fg=33)
    c.listme(items)



# TODO Update this one as to add many colours before writing the file.
def save_favs(save_as):
    fav_fg = []
    fg_resp = input("Favourite Col#\n")
    fav_fg.append(fg_resp)
    go = ["Col: "+ str(col) + "\n" for col in fav_fg]
    with open(save_as, "a") as fi:
        fi.writelines(go)
    print("file saved: ", save_as)

file = "manychrome/examples/save_my_colors.ini"
# save_favs(file)


def choose_color():
    for i in range(0, 256):
        print(f"\033[38;5;16;48;5;{i}m  Col: {i}  \033[0m", "", f"\033[38;5;255;48;5;{i}m  Col: {i}  \033[0m")



def listme(self, list, heading=None):
    v = [str(item) for item in list]
    if not heading:
        print(f"{"\n".join(v)}")
    else:
        print(f"\33[48;5;{self.fg}m{heading}\033[0m\n{"\n".join(v)}")



# Findme: the intention is to easily find placeholders and other items in text and make them stand out in the whole context of the text. This to quickly get an overview of the text pertaining to the words or fragments of interest in the big scheme of things.
from colorful import Colorful

class FindMe(dict, Colorful):
    def __missing__(self, key):
        return str(key)


    def showme(self, doc):  # Could name to makemepop
        """Add placeholders and alternative text and set the configuration of colors. Can also be used for invoicing, emails, etc to change placeholder text to something different."""
        for key, value in self.items():
            v = Colorful()
            v = self.styleme(value)
            self.update({key: v})
        print(doc.format_map(self))

    def reverseme(self, words):
        pass

    def mirrorme(self, words):
        bg = dict(bg="fg")
        print(f"abc {bg["bg"]} def")


    def but_why(self, doc):  # To find why, what, and how sections of text for text analysis, text edit, and text manipulation purposes.
        """
        a = "Get text"
        b = "Get list of why words from db"
        c = "Find the words in the doc"
        cd = "Split the sentences where the words are present and store this list"
        d = "wrap only the words in the doc so to be able to display the whole text with the sentences"
        e = "return the list with only the sentences, and second, return the text where the words have been highlighted (to be able to check the sentences in the full context)."
        """

        why_words = ["why", "because", "which means", "therefore"]
        r = [doc.partition(why) for why in why_words]
        print(r)

    def but_what(self, doc):
        what_words = ["list of", "what", "essentially"]
        print(what_words)


    def but_how(self, list, doc):
        how_words = ["how", "you do", "first take"]
        print(how_words)




if __name__ == "__main__":

    f = FindMe(
        ought="choose",
        should="can decide to",
        cannot="is ok for now to"
    )
    f.fg = 33
    f.bg = 220

    template = "We have an example where we {ought} to update a lot of the code to be better organised. It {cannot} be arranged like this. We {should} make it a priority."
    f.showme(template)



from colorful import Colorful

"""The intention of stylish is to provide CLI text styling also without color"""
class Stylish(dict, Colorful):
    def __missing__(self, key):
        return str(key)

    def findme(self, doc):
        for key, value in self.items():
            v = Colorful()
            v = self.styleme(value)
            self.update({key: v})
        print(doc.format_map(self))

    """The only reason for these below is to only have to type, 'italics' or 'bold' or whatever when wanting to print in that style. So that all that's needed is to remember what style one wants and can then just type that. Not sure why I didn't just add these to the class Colorful straight though..."""
    def italics(self, txt):
        v = Colorful()
        v.it = True
        v.fg = self.fg
        print(v.styleme(txt))

    def highlight(self, txt):
        v = Colorful()
        v.fg = self.fg
        v.bg = self.bg
        print(v.styleme(txt))

    def underline(self, txt):
        v = Colorful()
        v.ul = True
        v.fg = self.fg
        print(v.styleme(txt))

    def strikethrough(self, txt):
        v = Colorful()
        v.st = True
        v.fg = self.fg
        print(v.styleme(txt))

    def bold(self, txt):
        v = Colorful()
        v.bo = True
        v.fg = self.fg
        print(v.styleme(txt))

    def swap(self, txt):
        # This one is not quite working
        v = Colorful()
        v.sh = True
        v.fg = self.fg
        v.bg = self.bg
        print(v.styleme(txt))



if __name__ == "__main__":

    msg = "This text will be printed one way or another."
    s = Stylish()
    s.fg=89
    s.bg=75
    s.italics(msg)
    s.swap(msg)
    s.bold(msg)
    s.underline("Read This!")



# -*- coding: utf-8 -*-

# Learn more: https://github.com/NeurosystemsCo/manychrome/setup.py

from setuptools import setup

with open("README.rst") as f:
    readme = f.read()

with open("LICENSE") as f:
    license = f.read()

setup(
    name="manychrome",
    version="0.1.0",
    description="A simple package to colour and style your text printed on the CLI.",
    long_description=readme,
    url = "https://github.com/NeurosystemsCo/manychrome",
    license=license,
    author="Maria Wenner",
    author_email="mail@neurosystems.co",
    packages=['manychrome'],
    classifiers=[
    "Programming Language :: Python :: 3",
    "Operating System :: MacOS"
    ],
    python_requires='>=3.8',
)
